require.paths.unshift(__dirname + '/../lib');
require.paths.unshift(__dirname + '/../dep');

var assert = require('assert'),
    walk = require('walk');

var waitSeconds = 0.5;

var stepperTests = {
    'basic serial usage': function(done) {
        var stepper = new Stepper();
    
        // handle callbacks
        stepper.on('end', function(val) {
            assert.equal(val, 4);
            done();
        });
        stepper.on('error', function(err) {
            throw err;
        });
    
        // add steps
        stepper.add(function(err, val) {
            return 1;
        });
        stepper.add(function(err, val) {
            return 2;
        });
        stepper.add(function(err, val) {
            var self = this;
            setTimeout(function() {
                self();
            }, (waitSeconds * 1000));
        });
        stepper.add(function(err, val) {
            return 4;
        });
    
        // run steps
        stepper.walk();
    },
    'serial error': function(done) {
        var stepper = new Stepper();
    
        // handle callbacks
        stepper.on('end', done);
        stepper.on('error', function(err) {
            assert.equal(err.message, 'Unfunny serial error');
        });
    
        // add steps
        stepper.add(function(err, val) {
            return 1;
        });
        stepper.add(function(err, val) {
            var self = this;
            try {
                throw new Error('Unfunny serial error');
            } catch(err) {
                self(err, null);
                return;
            }
        });
        stepper.add(function(err, val) {
            if (err) throw err;
            var self = this;
            setTimeout(function() {
                self();
            }, (waitSeconds * 1000));
        });
        stepper.add(function(err, val) {
            if (err) throw err;
            return 4;
        });
    
        // run steps
        stepper.walk();
    },
    'basic parallel usage': function(done) {
        var grouper = new Grouper();
    
        // handle callbacks
        grouper.on('end', function(val) {
            assert.equal(_(val).compact().length, 3);
            done();
        });
        grouper.on('error', function(err) {
            throw err;
        });
    
        // setup group functions
        grouper.add(function(fn) {
            setTimeout(function() {
                fn(null, 1);
            }, (waitSeconds * 1000 * 3));
        });
        grouper.add(function(fn) {
            setTimeout(function() {
                fn(null, 2);
            }, (waitSeconds * 1000 * 2));
        });
        grouper.add(function(fn) {
            setTimeout(function() {
                fn(null, 3);
            }, (waitSeconds * 1000));
        });
    
        // run steps
        grouper.walk();
    },
    'parallel error': function(done) {
        var grouper = new Grouper();

        // handle callbacks
        grouper.on('end', done);
        grouper.on('error', function(err) {
            assert.equal(err.message, 'Unfunny parallel error');
            done();
        });

        // setup group functions
        grouper.add(function(fn) {
            setTimeout(function() {
                fn(null, 1);
            }, (waitSeconds * 1000 * 3));
        });
        grouper.add(function(fn) {
            setTimeout(function() {
                try {
                    throw new Error('Unfunny parallel error');
                } catch(err) {
                    fn(err, null);
                    return;
                }
                fn(null, 2);                
            }, (waitSeconds * 1000 * 2));
        });
        grouper.add(function(fn) {
            setTimeout(function() {
                fn(null, 3);
            }, (waitSeconds * 1000));
        });

        // run steps
        grouper.walk();
    }
};

module.exports = stepperTests;
